""" Object Path module for Python
	release 0.1
	by Jens Peter Grosen
	www.grosen.dk/py/op.html

	Use at your own risk. :-)
"""
import os.path, filecmp, shutil, re, UserString

def binfilecmp(a, b, *args):
	a, b = path(a), path(b)
	if not filecmp.cmp(a.str(), b.str()):
		return 0
	#check binary
	al, bl = a.readlines(), b.readlines()
	if len(al) != len(bl):
		return 0
	for i in xrange(len(al)):
		if al[i]!=bl[i]:
			del al, bl
			return 0
	del al, bl
	return 1

def bindircmp(a, b, *args):
	a, b = path(a), path(b)
	# can get to work keep getting "too many arg. exp 2 got 4
	# filecmp.cmpfiles(a.str(), b.str(), a.liststr())
	al, bl = a.list(), b.list()
	al.sort()
	bl.sort()
	if len(al) != len(bl):
		return 0
	
	for i in xrange(len(al)):
		ai, bi = al[i], bl[i]
		if ai.isdir() and bi.isdir():
			if not bindircmp(ai, bi):
				return 0
		elif ai.isfile() and bi.isfile():
			if not ai.cmp(bi):
				return 0
		else:
			return 0
	return 1

def dirremove(a, *args, **kv):
	shutil.rmtree(a.str(), *args, **kv)
	
__defaultfilecmp__ = binfilecmp
__defaultdircmp__ = bindircmp
__defaultdirremove__ = dirremove

def path(p="", *args):
	"""makes a new filepath, dirpath or generalpath if no file/dir was found"""
	if args:
		s = p
		for x in args:
			s = os.path.join(s, x)
		p = s

	if os.path.isdir(str(p)):
		return dirpath(p)
	elif os.path.isfile(str(p)):
		return filepath(p)
	else:
		return generalpath(p)
	
def newfile(p):
	"""creates a new file, closes it and returns a filepath to it"""
	f = open(p, "w")
	f.close()
	return filepath(p)

def newdir(p):
	"""creates a new directory and returns a dirpath to it"""
	os.makedirs(p)
	return dirpath(p)

class generalpath(UserString.UserString):
	"""general path is used when the path is not a filepath or dirpath e.g. empty paths \"\""""
	__path=""
	def __init__(self, path=""):
		self.data=str(path)
	def str(self):
		"""returns the path as a string"""
		return str(self.data)
	def path(self, *args):
		"""returns a new path, like op.path(self, args)"""
		return path(str(self.data), *args)
	def isfile(self):
		"""returns 1 if the path is a file"""
		return 0
	def isdir(self):
		"""retruns 1 if the path is a directory"""
		return 0
	def name(self):
		r"""returns only the name and not any directories it is in e.g. "name.txt" from C:tempname.txt"""
		return os.path.basename(self.str())
	def namenoext(self):
		r"""returns only the name without extension e.g. "name" from name.txt"""
		n, _ = os.path.splitext(os.path.basename(self.str()))
		return n
	def ext(self):
		"""return only the extension e.g. \".txt\" from name.txt"""
		_, n = os.path.splitext(self.str())
		return n
	def size(self):
		"""returns the size of the directory or file"""
		return 0
	def __repr__(self):
		return str(self.str())
	def __str__(self):
		return str(self.str())
	def __cmp__(self, other):
		if other == None:
			return 1
		if isinstance(other, generalpath):
			return self.str()!=other.str()
		else:
			return self.str()!=other
	def __hash__(self):
		return hash(self.str())
	def remove(self, *args, **kv):
		"""deletes the file or dir(incl subdirs) from the filesystem"""
		pass
	def cmp(self, to, *args):
		"""return 1 if the two files or directorys are binary equal"""
		return self.str()==to.str()
	def copy(self, to):
		pass

class dirpath(generalpath):
	def isdir(self):
		return 1
	def size(self):
		sum = 0
		for f in self.list():
			sum += f.size()
		return sum
	def _findinclude_(self, find, p):
		if find:
			#make a re that find the string
			#make sure that re wont find any special chars
			escaped = re.escape(find)
			#change any * back to the re (.*)="find any number of any char"
			changed = escaped.replace("\*", ".*")
			startOfLine = "^"
			endOfLine = "$"
			find = startOfLine+changed+endOfLine
			#print "finding:", find, "on:", p, "with:", re.match(str(find), str(p))
			if not re.match(str(find), str(p)):
				return 0
		return 1
		
	def list(self, find=None):
		"""returns a list of op.path for each of the files and directorys in self if they match find"""
		r=[]
		for f in os.listdir(self.str()):
			if not self._findinclude_(find, f):
				continue
			r.append(path(os.path.join(self.str(), f)))
		return r
	def listonlyfiles(self, find=None):
		"""return a list of op.path for each of the files in self"""
		return [x for x in self.list(find) if x.isfile()]
	def listonlydirs(self, find=None):
		"""return a list of op.path for each of the directorys in self"""
		return [x for x in self.list(find) if x.isdir()]
	def listall(self, find=None):
		"""like list but includes subdirectories"""
		r=self.listonlyfiles(find)
		for d in self.listonlydirs():
			if self._findinclude_(find, d):
				r.append(d)
			for x in d.listall(find):
				r.append(x)
		return r
	def listallonlyfiles(self, find=None):
		"""return a list of op.path for each of the files in self incl. subdirectorys"""
		return [x for x in self.listall(find) if x.isfile()]
	def listallonlydirs(self, find=None):
		"""return a list of op.path for each of the directorys in self incl. subdirectorys"""
		return [x for x in self.listall(find) if x.isdir()]
	def remove(self, *args, **kv):
		__defaultdirremove__(self, *args, **kv)
	def makedirs(self, to):
		"""make one or more new directorys from self and returns a new path to it"""
		name = os.path.join(self.str(), to)
		os.makedirs(name)
		return path(name)
	def cmp(self, to, *args):
		return __defaultdircmp__(self.str(), to.str(), *args)
	def copy(self, to, *args):
		# what to do about copytree symlinks?
		shutil.copytree(self.str(), os.path.join(to.str(), self.name()), *args)
	def empty(self, *args, **kv):
		r"""removes all files and directorys in the directory"""
		for f in self.list():
			f.remove(*args, **kv)
	


class filepath(generalpath):
	def isfile(self):
		return 1
	def dir(self):
		r"""returns the directory part of the name e.g. "c:\temp" from "c:\temp\file.txt"""
		return path(os.path.dirname(self.str()))
	def remove(self, *args, **kv):
		try:
			os.remove(self.str())
		except:
			pass
	def cmp(self, to, *args):
		return __defaultfilecmp__(self.str(), to.str(), *args)
	def size(self):
		return os.path.getsize(self.str())
	def readlines(self):
		"""opens the file, readlines it and closes it"""
		f = open(self.str())
		r = f.readlines()
		f.close()
		return r
	def writelines(self, lines):
		"""opens the file, appends the lines to it and closes it"""
		f = open(self.str(), "a")
		f.writelines(lines)
		f.close()
	def read(self):
		"""opens the file, read it and closes it"""
		f = open(self.str())
		r = f.read()
		f.close()
		return r
	def write(self, data):
		"""opens the file, appends to it and closes it"""
		f = open(self.str(), "a")
		f.write(data)
		f.close()
	def copy(self, to):
		shutil.copy2(self.str(), to.str())
	def empty(self):
		"""removes the contents of the file"""
		f = open(self.str(), "w")
		f.close()

if __name__ == "__main__":
	import optester, unittest
	reload(optester)
	unittest.TextTestRunner().run(optester.suite)
	pass
	
